#include "stdafx.h"
#include "GradientFillVisComponent.h"

CGradientFillVisComponent::CGradientFillVisComponent()
{
	// set up a map for friendly names for the gradient
	m_mapGradients.SetAt("Rainbow Cycle", "{2, 4} [[0xFF0000,0x000000][0x00FF00,0x000000][0x0000FF,0x000000][0xFF0000,0x000000]]");
	m_mapGradients.SetAt("Night Life", "{2, 3} [[0x4F2F4F,0xFF7C00][0xFF7C00,0x4F2F4F][0x4F2F4F,0xFF7C00]]");
	
	// set up a map for friendly names for the gradient speed
	m_mapGradientSpeeds.SetAt("Fast", "0.1");
	m_mapGradientSpeeds.SetAt("Normal", "0.005");
	m_mapGradientSpeeds.SetAt("Slow", "0.0005");

	// call this in the contstructor to load the defaults
	SetData(NULL);
}

CGradientFillVisComponent::~CGradientFillVisComponent()
{
}

int CGradientFillVisComponent::GetData(COMPONENT_PROPERTY_DATA_ARRAY & aryPropertyData)
{
	// add all our properties to the array
	aryPropertyData.Add("Gradient", m_GradientEx.SaveToString(), m_mapGradients);
	aryPropertyData.Add("Speed", m_fGradientSpeed, m_mapGradientSpeeds);

	return 0;
}

int CGradientFillVisComponent::SetData(const COMPONENT_PROPERTY_DATA_ARRAY * paryPropertyData)
{
	// set the defaults
	m_GradientEx.LoadFromString(m_mapGradients["Rainbow Cycle"]);
	m_fGradientSpeed = float(atof(m_mapGradientSpeeds["Normal"]));

	// load any settings passed in
	if (paryPropertyData)
	{
		for (int z = 0; z < paryPropertyData->GetSize(); z++)
		{
			COMPONENT_PROPERTY_DATA * pPropertyData = paryPropertyData->GetAt(z);

			if (pPropertyData->GetLabel() == "Gradient")
				m_GradientEx.LoadFromString(pPropertyData->GetValue(m_mapGradients));
			else if (pPropertyData->GetLabel() == "Speed")
				m_fGradientSpeed = float(atof(pPropertyData->GetValue(m_mapGradientSpeeds)));
		}
	}

	// set the initialized flag to false, so we re-initialize next draw
	m_bInitialized = FALSE;
	return 0;
}

int CGradientFillVisComponent::Initialize()
{
	// do any loading / initialization in here
	m_fGradientY = 0;

	// update the initialized flag
	m_bInitialized = TRUE;

	return 0;
}

int CGradientFillVisComponent::Draw(IVisBuffer * pBuffer, IVisData * pVisData)
{
	// initialize if necessary (when settings change, etc.)
	if (m_bInitialized == FALSE)
		Initialize();

	// create a buffer helper, and use it (provides a lot of fast, handy fuctions)
	CVisBufferHelper VisBuffer(pBuffer);

	// build a lookup table the height of the buffer (each row is a new color)
	m_GradientEx.BuildQuickLookupTable(m_fGradientY, VisBuffer.GetHeight());

	// loop through each row of the display
	for (int y = 0; y < VisBuffer.GetHeight(); y++)
	{
		// get the color for this row
		const unsigned int rgbRowColor = m_GradientEx.m_paryQuickLookupTable[y];

		// set all the pixels in that row to this color
		for (int x = 0; x < VisBuffer.GetWidth(); x++)
		{
			VisBuffer.SetPixel(x, y, rgbRowColor);
		}

		/* // faster method of above loop... just to illustrate
		unsigned int * pRowStart = VisBuffer.GetPixelPointer(0, y);
		int x = VisBuffer.GetWidth();
		while (x--)
			*pRowStart++ = rgbRowColor;
		*/
	}

	// increment the gradient position (reset to zero if we reached the end of the gradient)
	m_fGradientY += m_fGradientSpeed;
	if (m_fGradientY > float(1.0)) m_fGradientY = 0;

	return 0;
}