#ifndef _visualizationbufferhelper_h_
#define _visualizationbufferhelper_h_

#include "IVisBuffer.h"

class CVisBufferHelper
{
public:

	// construction
	CVisBufferHelper(IVisBuffer * pVisBuffer)
	{
		m_pVisBuffer = pVisBuffer;
		m_sizeBuffer = m_pVisBuffer->GetDimensions();
		m_nWidthBytes = m_sizeBuffer.cx * 4;
		m_ptCenter.x = (m_sizeBuffer.cx + 1) / 2;
		m_ptCenter.y = (m_sizeBuffer.cy + 1) / 2;
		GetCurrentBuffer();
	}

	// wrap the IVisBuffer interface for ease
	unsigned char * GetBuffer(int nOffset = 0) { return m_pVisBuffer->GetBuffer(nOffset); }
	virtual HDC GetHDC(int nOffset = 0) { return m_pVisBuffer->GetHDC(nOffset); }
	virtual CSize GetDimensions() { return m_sizeBuffer; }
	virtual int FlipBuffer() 
	{
		int nRetVal = m_pVisBuffer->FlipBuffer(); 
		GetCurrentBuffer();
		return nRetVal;
	}

	// get the current buffer (call this again if the buffer is flipped, etc.)
	void GetCurrentBuffer()
	{
		m_pData = m_pVisBuffer->GetBuffer();
	}

	// data
	IVisBuffer * m_pVisBuffer;
	unsigned char * m_pData;
	CSize m_sizeBuffer;
	CPoint m_ptCenter;
	int m_nWidthBytes;

	// helper functions (all inlined for speed)
	__inline int GetWidth()
	{
		return m_sizeBuffer.cx;
	}

	__inline int GetHeight()
	{
		return m_sizeBuffer.cy;
	}

	__inline unsigned int GetTotalBytes()
	{
		return m_nWidthBytes * m_sizeBuffer.cy;
	}
	
	__inline unsigned char * GetMaximumPointer()
	{
		return &m_pData[GetTotalBytes()];
	}
	
	__inline unsigned char * GetClippedPointer(unsigned char * pBuffer)
	{
		unsigned char * pClipped = pBuffer;
		unsigned char * pMin = m_pData;
		unsigned char * pMax = GetMaximumPointer();
		if (pClipped < pMin) pClipped = pMin;
		if (pClipped > pMax) pClipped = pMax;
		return pClipped;
	}
	
	__inline unsigned char * GetXYPointer(int X, int Y)
	{
		if (X < 0) X = 0;
		else if (X > (m_sizeBuffer.cx - 1)) X = (m_sizeBuffer.cx - 1);

		if (Y < 0) Y = 0;
		else if (Y > (m_sizeBuffer.cy - 1)) Y = (m_sizeBuffer.cy - 1);
		
		return &m_pData[((m_sizeBuffer.cy - 1 - Y) * m_nWidthBytes) + (X * 4)];
	}

	__inline unsigned int * GetPixelPointer(int X, int Y)
	{
		return (unsigned int *) GetXYPointer(X, Y);
	}

	__inline void SetPixel(int X, int Y, unsigned int rgbColor)
	{
		*GetPixelPointer(X, Y) = rgbColor;
	}

	__inline int GetCenterX()
	{
		return (GetDimensions().cx + 1) / 2;
	}
	
	__inline int GetCenterY()
	{
		return (GetDimensions().cy + 1) / 2;
	}
	
	__inline int GetCornerRadius()
	{
		CSize sizeBuffer = GetDimensions();
		float CenterX = float(sizeBuffer.cx) / 2;
		float CenterY = float(sizeBuffer.cy) / 2;
		return (int) (sqrt(CenterX * CenterX + CenterY * CenterY) + 1);
	}
	
	__inline int GetMinimumDimensionRadius()
	{
		int nCenterX = (m_sizeBuffer.cx + 1) / 2;
		int nCenterY = (m_sizeBuffer.cy + 1) / 2;
		return (nCenterX > nCenterY) ? nCenterY : nCenterX;
	}
	
	__inline int GetMaximumDimensionRadius()
	{
		int nCenterX = (m_sizeBuffer.cx + 1) / 2;
		int nCenterY = (m_sizeBuffer.cy + 1) / 2;
		return (nCenterX > nCenterY) ? nCenterX : nCenterY;
	}

	__inline int GetWidthBytes()
	{
		return m_nWidthBytes;
	}
};

#endif // _visualizationbufferhelper_h_