#if !defined(AFX_DRAWCOMPONENT_H__29E798BC_1DB4_46C2_84DE_DB4365F00A28__INCLUDED_)
#define AFX_DRAWCOMPONENT_H__29E798BC_1DB4_46C2_84DE_DB4365F00A28__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MJVisLib.h"

class IVisData;

// data type for property list
#define PIT_COMBO	0
#define PIT_EDIT	1
#define PIT_BUTTON	2
#define PIT_DROPLIST 3

enum COMPONENT_TYPES
{
	COMPONENT_TYPE_TRANSFORM,
	COMPONENT_TYPE_DRAW,
	COMPONENT_TYPE_INFORMATION,
	COMPONENT_TYPE_OTHER,
};

struct VISUALIZATION_REGISTRATION_INFO
{
	char cName[256];
	char cVersion[256];
	char cDescription[256];
	char cCopyright[256];
	char cWebsite[256];
};

struct VISUALIZATION_COMPONENT_INFO
{
	char cGUID[256];
	unsigned long nVersion;
	COMPONENT_TYPES ComponentType;
	char cComponentName[256];
	char cComponentDescription[4096];
};

class COMPONENT_PROPERTY_DATA
{
public:
	COMPONENT_PROPERTY_DATA()
	{
		bstrLabel = NULL;
		bstrValue = NULL;
		bstrValueChoices = NULL;
		nDataType = PIT_EDIT;
	}

	~COMPONENT_PROPERTY_DATA()
	{
		Empty();
	}

	void Empty()
	{
		SAFE_FREE_STRING(bstrLabel)
		SAFE_FREE_STRING(bstrValue)
		SAFE_FREE_STRING(bstrValueChoices)
		nDataType = PIT_EDIT;
	}

	void Assign(LPCSTR lpszLabel, LPCSTR lpszValue, LPCSTR lpszValueChoices, int a_nDataType)
	{
		Empty();
		bstrLabel = CString(lpszLabel).AllocSysString();
		bstrValue = CString(lpszValue).AllocSysString();
		bstrValueChoices = CString(lpszValueChoices).AllocSysString();
		nDataType = a_nDataType;
	}

	CString GetLabel() { return CString(bstrLabel); }
	CString GetValueChoices() { return CString(bstrValueChoices); }
	int GetDataType() { return nDataType; }
	CString GetValue() { return CString(bstrValue); }
	CString GetValue(CMapStringToString & mapPresets)
	{
		CString strValue(bstrValue);
		CString strTemp(bstrValue);
		mapPresets.Lookup(strValue, strTemp); 
		return strTemp;
	}

	CString & GetWorkingValue() 
	{ 
		return strWorkingValue;
	}
	void SaveWorkingValue()
	{
		SAFE_FREE_STRING(bstrValue)
		bstrValue = strWorkingValue.AllocSysString();
	}

private:

	BSTR bstrLabel;
	BSTR bstrValue;
	BSTR bstrValueChoices;
	CString strWorkingValue;
	long nDataType;
};

class COMPONENT_PROPERTY_DATA_ARRAY
{
public:

	// construction / destruction
	COMPONENT_PROPERTY_DATA_ARRAY();
	~COMPONENT_PROPERTY_DATA_ARRAY();
	
	// CArray like functionality
	COMPONENT_PROPERTY_DATA * operator[](int nIndex) const
	{
		return (COMPONENT_PROPERTY_DATA *) &m_aryProperties[nIndex];
	}
	COMPONENT_PROPERTY_DATA * GetAt(int nIndex) const
	{
		return (COMPONENT_PROPERTY_DATA *) &m_aryProperties[nIndex];
	}
	int GetSize() const { return m_nTotalProperties; }
	
	// data
	BOOL m_bFriendlyNames;

	// functions
	void RemoveAllSafe();
	//int Add(COMPONENT_PROPERTY_DATA * newElement);
	void Add(CString strLabel, CString strValue, long nDataType = PIT_EDIT, CString strValueChoices = "");
	void Add(CString strLabel, int nValue, long nDataType = PIT_EDIT, CString strValueChoices = "");
	void Add(CString strLabel, bool bValue, long nDataType = PIT_EDIT, CString strValueChoices = "");
	void Add(CString strLabel, float fValue, long nDataType = PIT_EDIT, CString strValueChoices = "");
	void Add(CString strLabel, int nValue, CMapStringToString & mapPresets);
	void Add(CString strLabel, float fValue, CMapStringToString & mapPresets);
	void Add(CString strLabel, CString strValue, CMapStringToString & mapPresets);

private:
	
	COMPONENT_PROPERTY_DATA m_aryProperties[1024];
	int m_nTotalProperties;
};

class CVisGenerateHelper
{
public:
	unsigned long rgbPrimaryColor;
	unsigned long rgbSecondaryColor1;
	unsigned long rgbSecondaryColor2;
	unsigned long rgbSecondaryColor3;
};

class CVisComponent  
{
public:
	CVisComponent();
	virtual ~CVisComponent();

	virtual int GetData(COMPONENT_PROPERTY_DATA_ARRAY & aryPropertyData) { return 0; }
	virtual int SetData(const COMPONENT_PROPERTY_DATA_ARRAY * paryPropertyData = NULL) { return 0; }
	virtual int Generate(CVisGenerateHelper * pGenerateInfo) { return 0; }

	virtual int Draw(IVisBuffer * pBuffer, IVisData * pVisData) = 0;

	virtual int InitializeWindow(CRect rectWindow) { return 0; }
	virtual int TrackChange() { return 0; }
	
	// filled in by visualization engine... ignore
	char m_cGUID[256];
	char m_cComponentName[256];

protected:

	virtual int Initialize() { return 0; }
	
	CVisCommonFunctions	m_Helper;
	BOOL				m_bInitialized;
};

// proc's for the DLL
typedef void (* proc_GetRegistrationInfo)(VISUALIZATION_REGISTRATION_INFO *); 
typedef int (* proc_GetComponentInfo)(int, VISUALIZATION_COMPONENT_INFO *); 
typedef int (* proc_CreateComponent)(int, CVisComponent **); 

#endif // !defined(AFX_DRAWCOMPONENT_H__29E798BC_1DB4_46C2_84DE_DB4365F00A28__INCLUDED_)
